'0015.BAS:  HD4478-kompatible LCD-Anzeige
'          --->korrigierte Version, beim Original waren die LCD-Pins 4+5 vertauscht.
'              Damit blieb RS permanent auf Masse.
'Hardware: LCD-Anzeige mit HD4478-kompatibler Schnittstelle
'          LCD-Verbindungen:
'          LCD01 an Masse (VSS)
'          LCD02 an +5V   (VDD)
'          LCD03 an Poti2 (V0)
'          LCD04 an PD5   (RS)
'          LCD05 an PD6   (R/W)  --> bleibt permanent auf Masse (0V)
'          LCD06 an PB0   (E)
'          LCD11 an PB1   (DB4)
'          LCD12 an PB2   (DB5)
'          LCD13 an PB3   (DB6)
'          LCD14 an PB4   (DB7)

'Der RW-Eingang der LCD-Anzeige (steuert die Datenrichtung) wird von Bascom
'nicht verwendet und *mu* auf Masse liegen, damit die LCD-Anzeige auf die
'Signale reagiert. Da wir auf dem Experimentierboard keine Masse-Buchse haben,
'mibrauchen wir einfach Pin PD6, den wir auf LOW setzen (der Pin kann narlich
'sinnvoll bei eventuellen Erweiterungen genutzt werden).
'
'Wichtig: Die LPT-Stromversorgung kann fr die LCD nicht verwendet werden, weil
'         der LCD-Controller HD4478 viel Strom zieht (15 bis 30mA).
'         Also bitte externe Stromversorgung nehmen und nicht vergessen, in den
'         WinAVR-Einstellung die LPT-Stromversorgung auszuschalten (sonst setzt
'         WinAVR die Stromversorgungsleitungen nach Ende der Programmierung).
'
'Funktion:
'Man gibt im Terminalprogramm ein oder mehrere Kommando-Bytes ein und sendet
'sie. Einde Liste der Kommandos ist in der Hauptschleife (Main:). Zum Beispiel:
'    15Hallo   (1:InitLcd, 2:Cls, "Hallo": Gibt Text Hallo aus)
'Mit dem Terminalprogramm lassen sich die LCD-Befehle sehr schn einzelnen
'testen und man kann alles mgliche ausprobieren, ohne stndig das Programm neu
'laden zu mssen.
'-------------------------------------------------------------------------------
$Regfile = "2313def.dat"
$Crystal = 3686400  '3,6864MHz-Quarz
$Baud    = 9600

Dim sBuffer As String*20 At &H60
Dim bFirstByte As Byte At &H60 Overlay

On URXC OnRxUART    'Interrupt-Routine
Enable URXC         'Capture-Interrupt
Enable Interrupts   'Interrupts global

DDRD.6  = 1  'Datenrichtung: PD6 soll Ausgang sein (LCD-Anschlu R/W)
PORTD.6 = 0  'Pin PD6 auf LOW setzen (nicht verwendet, MU aber LOW sein)

Config Lcd=16*2 'Wir haben ein LCD-Display mit 16 Zeichen in 2 Zeilen
Config LcdPin=Pin,DB4=PortB.1,DB5=PortB.2,DB6=PortB.3,DB7=PortB.4,E=PortB.0,RS=PortD.5

Main:               'Hauptschleife
  If Len(sBuffer)>0 Then  'Falls (noch) Zeichen im UART-Puffer sind...
    'Die folgenden Kommandozeichen am besten auf dem Ziffernblock eingeben!
    'Alle Nicht-Zahlen werden entsprechend des LCD-Zeichensatzes ausgegeben.
    Select Case bFirstByte         'Erstes Byte des UART-Puffers auswhlen
    Case "1":  InitLcd             'Bitte einmalig nach Reset aufrufen
    Case "3":  GoTo Fun            'Ausgabe selbstdefinierter Zeichen
    Case "5":  Cls                 'Lscht die komplette Anzeige
    Case "7":  Cursor On
    Case "9":  Cursor Off          'Cursor Blink/NoBlink gibt es auch
    Case "8":  Home Upper          'Obere Zeile (oder auch UpperLine)
    Case "2":  Home Lower          'Untere Zeile (oder auch LowerLine)
    Case "4":  ShiftCursor Left    'Shift um ein Zeichen nach links
    Case "6":  ShiftCursor Right   'Shift um ein Zeichen nach rechts
    Case Else: Lcd Chr(bFirstByte) 'Zeichen an aktueller Position ausgeben
    End Select
    sBuffer=Mid(sBuffer,2)         'Kommando aus dem Puffer entfernen
    GoTo EndFun
    Fun:
      Deflcdchar 0,4,14,31,16,31,14,4,32   'Zeichen 0 (mit LCD-Designer erzeugt)
      Deflcdchar 1,4,14,31,16,24,28,14,32  'Zeichen 1 (mit LCD-Designer erzeugt)
      Cursor Off
      Cls 'LCD-Daten-RAM anwhlen
      Do
        Lcd Chr(0) 'Selbstdefiniertes Zeichen Nr. 0 darstellen
        ShiftCursor Left
        Lcd Chr(1) 'Selbstdefiniertes Zeichen Nr. 1 darstellen
        ShiftCursor Left
       Loop
    EndFun:
    'Weitere anwendbare LCD-Befehle sind: Display On/Off, Locate, DefLcdChar
  End If
Goto Main
'-------------------------------------------------------------------------------
OnRxUART:          'Zeichen via UART eingetroffen
  sBuffer=sBuffer+Chr(UDR) 'Zeichen aus der UART in den Puffer geben
  Printbin UDR  'Echo
Return