'FQ02_256.bas:    Frequenzzhler 30MHz...500MHz/1GHz mit LCD-Anzeige
'                 Etwas ausgebautere Variante, Vorteilung fest eingestellt
'                 (exakt wie FQ02.BAS, aber Vorteilung 1/256 eingestellt)
'Hardware:        AT90S2313 mit 16,384MHz-Quarz (alternativ: 3,2768MHz)
'                 T1/PD5: Signaleingang vom Vorteiler U813BS
'                         (alternativ der ltere 1/64-Vorteiler U664BS)
'                 PD6: Ausgang zum Einstellen der Vorteilung beim U813BS
'                 LCD-Anzeige an PortB, Pin-Belegung siehe unten
'Entwickler:      Roland Walter
'Letzte nderung: 10.10.2002  (DD.MM.YYYY)
'-------------------------------------------------------------------------------
'Funktion:
'Das Eingangssignal wird mit einen U813BS durch 128 vorgeteilt und ber einen
'NPN-Transistor-ECL-zu-TTL-Wandler an den Timer1-Eingang des AVR gegeben.
'Die Teilung 1/128 ist ntig, wenn der Zhler bis gut 1GHz arbeiten soll (der
'U813BS kan 1/64, 1/128 und 1/256 teilen). Der etwas ltere U664B kann nur 1/64
'teilen. Die hchste Zhlfrequenz ist im Prinzip 1/2 AVR-Taktfrequenz oder ge-
'nauer gesagt mu jede Halbwelle des Eingangssignals mindestens so gro sein,
'wie eine volle AVR-Takt-Periode. Bei einem symmetrischen Eingangssignal (posi-
'tive und negative Halbwelle gleich lang) ist die Hchste Zhlfrequenz also:
'AVR-Takt      Vorteilung    Maximalfrequenz
'16,384MHz/2   256           2,096GHz (real: 1,3GHz, der U813BS kann nicht mehr)
'16,384MHz/2   128           1,048GHz
'16,384MHz/2   64            524MHz
'16,384MHz/2   1             8,1MHz (ohne Vorteiler)
'3,2768MHz/2   256           419MHz
'3,2768MHz/2   128           209MHz
'3,2768MHz/2   64            104MHz
'Mit dem 8-Bit-Timer0 wird die Torzeit gesteuert und der 16-Bit-Timer1 arbeitet
'beim geffneten Tor als Counter. Das ist eigentlich schon alles. Der Rest ist
'etwas Binr-Mathematik, dann etwas String-Formatierung und Ausgabe an die LCD.
'Die Torzeit wird durch Timer0-berlufe gesteuert. Im einfachsten Fall wird das
'Tor bei jedem Timer0-berlauf auf- oder zugemacht. Man kann die Torzeit aber
'auch verlngern, indem man mehrere Timer0-berlaufe abwartet. Bei Timer1 (dem
'eigentlichen Zhler wird mit einem 32-Bit-Integer gearbeitet. Bei jedem Timer1-
'berlauf wird das HighWord um 1 erhht. Nach Schlieen des Tores wird dann noch
'der aktuelle Inhalt von Timer1 als LowWord hinzuaddiert.
'Die Quarzfrequenz ist alles andere als egal. Die folgenden beiden Punkte sind
'von ganz zentraler Bedeutung:
'1. Die Taktfrequenz sollte so hoch wie mglich sein. Wenn man Timer1 mit einem
'   externen Signal taktet (Zhlerbetrieb), dann mu JEDE der beiden Halbwellen
'   mindestens so lang sein wie eine Periodendauer das AVR-Takts.
'2. Wir bentigen einen Dezimalwert als Ergebnis. Deshalb mu die Torzeit
'   "dezimal lang" sein, also z.B. 1sec, 1/10sec, 1/100sec, denn eine normale
'   Divisions-Operation ist im AVR nicht implementiert und mte aufwendig durch
'   eine Software-Division gelst werden. Was aber sehr effizient geht, ist die
'   Division und Multiplikation mal/durch 1,2,4,8,16,32,64 (...), denn diese
'   kann durch einfaches Shiften des Werts erledigt werden. Damit sind also auch
'   Torzeiten wie z.B. 1/2 oder 1/64 Sekunde mglich.
'   Die Dezimalkommastelle darf im Integer-Ergebnis verschoben sein, denn das
'   knnen wir im Nachhinein im Ergebnis-String korrigieren.
'Als Beispiel folgen zwei Quarzfrequenzen (beide billige Massenware), an denen
'das Vorgehen dargestellt wird:
'Wir nehmen den 8-Bit-Timer0 fr die Torzeit. Fr die Torzeit wird Timer0 durch
'1024 geteilt und das Tor wird bei jedem Timer0-berlauf (also alle 256 Timer0-
'takte) geffnet/geschlossen. Die Torzeit kann auerdem in 256er-Schritten ver-
'lngert werden, indem man Timer0 mehrfach (z.B. 8x) berlaufen lt, bevor man
'den Zhlerstand ausliest. Das ergibt weitere Variationsmglichkeiten.
'
'3,276.800MHz-Quarz (kurze Torzeit, dafr ungenauer):
'       1. Clock-Vorteilung 1/1024: 3,2768MHz/1024=3200Hz
'       2. Tor bei Timer0-Overflow: 3200Hz/256=12,5Hz=0,08Sec
'       3. Zhlerstand/8: Shift CountVal,Right,3 ==> Entspricht Torzeit=10mSec
'3,276.800MHz-Quarz (lngere Torzeit durch 8 Timer0-berlufe, dafr genauer):
'       1. Clock-Vorteilung 1/1024: 3,2768MHz/1024=3200Hz
'       2. Tor ist 8 mal Timer0-Overflow: 3200Hz/256/8=1,5625Hz=0,64Sec
'          ==> Entspricht Torzeit=10mSec, verrechnet sich bei 1/64-Vorteilung!!!
'16,384MHz-Quarz (kurze Torzeit, dafr ungenauer):
'       1. Clock-Vorteilung 1/64: 16,384MHz/64=256000Hz
'       2. Tor bei Timer0-Overflow: 256000Hz/256=1000Hz=0,001Sec ==> Entspricht Torzeit=1mSec
'16,384MHz-Quarz (lngere Torzeit, dafr genauer):
'       1. Clock-Vorteilung 1/1024: 16,384MHz/1024=16000Hz
'       2. Tor bei Timer0-Overflow: 16000Hz/256=62,5Hz=0,016Sec
'       3. Zhlerstand/16: Shift CountVal,Right,4 ==> Entspricht Torzeit=10mSec
'16,384MHz-Quarz (noch lngere Torzeit, dafr noch genauer):
'       1. Clock-Vorteilung 1/1024: 16,384MHz/1024=16000Hz
'       2. Tor bei Timer0-Overflow: 16000Hz/256=62,5Hz=0,016Sec
'       2. Torzeit mal 10 verlngern: 0,016Sec*40=0,64sec
'       3. Hebt sich auf mit der Vorteilung 1/64 ==> Entspricht Torzeit=10mSec
'
'Mit gut 16MHz wird der AT90S2313 natrlich bertaktet (10MHz sind laut Daten-
'blatt maximal zulssig). Trotzdem habe ich noch nie von Schwierigkeiten bei
'dieser Frequenz gehrt - im Gegenteil: Ein industrieller Hersteller hat berich-
'tet, da bei etlichen Tausend Exemplaren nicht ein einziger AVR Probleme mit
'16MHz Taktfrequenz hatte. Einzig der EEPROM kann bei dieser Frequenz Probleme
'machen, aber den brauchen wir hier nicht. Auerdem kann es sein, da die SPI-
'Programmierung des AT90S2313 mit einer geringeren Taktfrequenz erfolgen mu.
'Aber das sind die einzigen bekannten "Schwachstellen". Atmel hatte die ersten
'AVRs brigens mit 20MHz spezifiziert und nur aufgrund von Schwierigkeiten mit
'dem EEPROM "nach unten korrigiert". Das nur zur Beruhigung.
'-------------------------------------------------------------------------------
'Hardware: LCD-Anzeige mit HD4478-kompatibler Schnittstelle
'          LCD-Verbindungen:
'          LCD01 an Masse (VSS)
'          LCD02 an +5V   (VDD)
'          LCD03 an Poti  (V0)
'          LCD04 an PB6   (RS)
'          LCD05 an PB5   (R/W)  --> wird permanent auf Masse (0V) gelegt
'          LCD06 an PB4   (E)
'          LCD11 an PB3   (DB4)
'          LCD12 an PB2   (DB5)
'          LCD13 an PB1   (DB6)
'          LCD14 an PB0   (DB7)
'Der RW-Eingang der LCD-Anzeige (steuert die Datenrichtung) wird von Bascom
'nicht verwendet und *mu* auf Masse liegen, damit die LCD-Anzeige auf die
'Signale reagiert. Da wir auf dem Experimentierboard keine Masse-Buchse haben,
'mibrauchen wir einfach Pin PD5, den wir auf LOW setzen (der Pin kann narlich
'sinnvoll bei eventuellen Erweiterungen genutzt werden).
'
'Wichtig: Die LPT-Stromversorgung kann fr die LCD nicht verwendet werden, weil
'         der LCD-Controller HD4478 viel Strom zieht (15 bis 30mA).
'         Also bitte externe Stromversorgung nehmen und nicht vergessen, in den
'         WinAVR-Einstellung die LPT-Stromversorgung auszuschalten (sonst setzt
'         WinAVR die Stromversorgungsleitungen nach Ende der Programmierung).
'
'-------------------------------------------------------------------------------
$Regfile = "2313def.dat"
'$Crystal = 3276800        'Quarz: 3,276.800MHz (NICHT 3.6864 MHz!!!)
$Crystal = 16384000        'Quarz: 16,384MHz
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Dim lCount   As Long At &H60              'Zhler-Variable
Dim wCountLo As Word At &H60 Overlay      'LoWord der Zhlervariablen
Dim wCountHi As Word At &H62 Overlay      'HiWord der Zhlervariablen
Dim sResult As String*16 At &H64          'Ausgabestring
Dim Result(16) As Byte At &H64 Overlay    'Ausgabestring als Byte-Array
Dim sTempCount As String*10 At &H75       'Mu Len(Str(&HFFFFFFFF)) lang sein
Dim TempCount(10) As Byte At &H75 Overlay 'Temp-String als Byte-Array
Dim i As Byte
Dim Gate As Byte
'
'Timer0 (Torzeit) und Timer1 (Zhler) konfigurieren:
On Timer0 OnGateTime             'Interrupt-Routine fr die Torzeit
On Timer1 OnCounterOverflow      'Interrupt-Routine frs HiWord des Zhlerstands
Config Timer0 = Timer , Prescale = 1024  'Torzeit-Timer mit Takt/1024
Config Timer1 = Counter , Edge = Rising  'Zhler an steigender Signalflanke
DDRD.5  = 0  'PD5/T1 auf Eingang setzen (Zhler-Eingang)
DDRD.6  = 1  'PD6: Stellt U813BS-Vorteilung ein, mehr siehe nchste Zeile
PORTD.6 = 0  'Vorteilung: DDRD.6=0 + PORTD.6=0: High-Z ==> Vorteilung 1/64
'             Vorteilung: DDRD.6=1 + PORTD.6=1: High   ==> Vorteilung 1/128
'             Vorteilung: DDRD.6=1 + PORTD.6=0: Low    ==> Vorteilung 1/256
'             Achtung: Beim U664B nur High-Z und nichts andres!
'
'LCD-Anzeige konfigurieren:
Config Lcd = 16 * 2 'Wir haben ein LCD-Display mit 16 Zeichen in 2 Zeilen
Config LcdPin = Pin , DB4 = PortB.3 , DB5 = PortB.2 , DB6 = PortB.1 , DB7 = PortB.0 , E = PortB.4 , RS = PortB.6
DDRB.5  = 1  'PB5 auf Ausgang setzen (gibt 0 Volt an LCD-Anschlu R/W aus)
PORTB.5 = 0  'Pin PB5 auf LOW setzen (mu permanent 0 Volt fhren!)
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Cursor Off   'LCD-Kursor ausschalten
'Begrungstext auf der LCD ausgeben (kann bei Platzmangel natrlich entfallen):
Home Upper
sResult = "Frequenzz" + Chr(&HE1) + "hler  "
Lcd sResult
Home Lower
Lcd "30MHz ... 1GHz  "
WaitMs 1500
Home Upper
For i=1 To 16   'Ansteigender Balken
  Lcd Chr(255)  'Schwarzes Karo
  WaitMs 30
Next i
Home Lower
For i=1 To 16   'Ansteigender Balken
  Lcd Chr(255)  'Schwarzes Karo
  WaitMs 30
Next i
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
Enable Timer0       'Timer0-Interrupts ein (Torzeit)
Enable Interrupts   'Interrupts global einschalten
Start Timer0

Main:
Goto Main
'-------------------------------------------------------------------------------
'LCD-Sonderzeichen zur Anzeige des eingestellten Frequenzbereichs (Vorteilung):
'Deflcdchar 0,10,21,17,32,1,14,17,14 ' "1,3" senkrecht
'Deflcdchar 1,14,17,14,32,1,32,31,8  ' "1,0" senkrecht
'Deflcdchar 2,18,21,29,32,1,14,17,14 ' "0,5" senkrecht
'Deflcdchar 3,32,32,32,32,6,21,17,14 ' "G" senkrecht
'Deflcdchar 4,32,32,31,32,6,21,17,14 ' "G" Nr. 1 senkrecht
'Deflcdchar 5,32,31,32,32,6,21,17,14 ' "G" Nr. 2 senkrecht
'Deflcdchar 6,31,32,32,32,6,21,17,14 ' "G" Nr. 3 senkrecht

OnGateTime:               'Timer0-Torzeit beendet
  If Gate = 0 Then        'Zhler starten
    Enable Counter1       'Zhler-Overflow-Interrupts einschalten
    Start Counter1
    Gate = 1
  ElseIf Gate < 40 Then   '40 Timer0-Umlufe warten
    Incr Gate
  Else                    'Tor zumachen, Zhler auslesen
    Stop Counter1
    Disable Counter1      'Zhler-Overflow-Interrupts ausschalten
    wCountLo = Counter1   'LoWord in die LONG-Zhlervariable holen
    Counter1 = 0          'Zhler zurcksetzen (Voreinstellen geht hier auch!)
    Gate = 0
    '
    'Jetzt mssen wir den aktuellen Zhlerstand, die Torzeit und die Vorteilung
    'zum echten dezimalen Zhlerstand umrechnen:
    'Bei 0,64ms Torzeit und Vorteilung 1/64 mu hier nichts umgerechnet werden.
    'Bei 1/128 und 1/256 ist die Umrechnung wie folgt:
    'Shift lCount , Left , 1 'lCount=lCount*2 (bei 16,384MHz und 1/128)
    Shift lCount , Left , 2 'lCount=lCount*4 (bei 16,384MHz und 1/256)
    sTempCount = Str(lCount)   'Wert in String umwandeln
    'Jetzt den 16 Zeichen langen String mit Hilfe des Overlay-Felds formatieren:
    'Hier ist brigens nichts optimiert. Auerdem kann die String-Wandlung bei
    'Bedarf auch codesparender gestaltet werden, wenn man sich dazu eine spe-
    'ziell angepate Assembler-Routine schreibt. Ein Beispiel dafr ist in der
    'Datei MCS.LIB zu finden [_Byte2Str].
    sResult = Space(16)
    Result(16) = &H20    'Das letzte Zeichen ist immer ein Leerzeichen
    Result(15) = &H20    'Das vorletzte Zeichen ist immer ein Leerzeichen
    i = Len(sTempCount) 'Aktuelle Position im Quell-String
    Result(14) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(13) = &H2E   'ASC(&H2E)="."
    Result(12) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(11) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(10) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(9) = &H2E   'ASC(&H2E)="."
    Result(8) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(7) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(6) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(5) = &H2E   'ASC(&H2E)="."
    Result(4) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(3) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    Result(2) = TempCount(i) : Decr i
    If i < 1 Then Goto DoneString
    DoneString:
    '
    lCount = 0             'Zhler-Variable zurcksetzen (Voreinstellen geht hier auch!)
    Home Upper             'In die obere LCD-Zeile gehen
    Lcd sResult            'Exakt 16 Zeichen lang
    Home Lower             'In die untere LCD-Zeile gehen
    Lcd " GHz MHz KHz Hz " 'Exakt 16 Zeichen lang
    '
    'Fr die LCD-Anzeige ist inzwischen SEHR viel Zeit vergangen. Inzwischen
    'eingetroffene Timer-Interruptanforderungen sind daher lngst veraltet und
    'wrden uns falsche Zhlerstnde bescheren. Deshalb lschen wir jetzt even-
    'tuell vorhandene Interruptanforderungen:
    TIFR.TOV0 = 1 'Timer0-Overflow-Flag lschen (nicht = 0!!!)
    TIFR.TOV1 = 1 'Timer1-Overflow-Flag lschen (nicht = 0!!!)
  End If
Return
'-------------------------------------------------------------------------------
OnCounterOverflow:
  Incr wCountHi    'HiWord der Zhlervariablen erhhen (lCount=lCount+65536)
Return
'-------------------------------------------------------------------------------