'Mega169_T4.bas: AVR-Butterfly (ATmega169-basiert) mit Bascom 1.11.7.4
'
'Demonstriert, wie der momentan noch nicht untersttzte ATmega169 durch Vortuschen des ATmega16
'programmiert werden kann ("T" wie Trick).
'Dieses Beispiel-Listing steuert ein paar Segmente der LCD-Anzeige an und verwendet dabei den Weg ber
'Overlay-Variablen im SRAM-Bereich zwischen SRAM-Adresse &H20 bis &H5F. Das ist der STRAM-Bereich, in
'den die I/O-Register gemappt werden. Dieser Weg ist wohl mit Abstand der gnstigste. Zum einen ist
'die sptere Umstellung auf "echten ATmega169-Bascom-Support" ein Kinderspiel und zum anderen lassen
'sich die I/O-Register auf diese Weise genau so ansprechen, wie es in Bascom mglich ist, sobald der
'ATmega169 direkt von Bascom untersttzt wird. Das gilt auch fr das setzen einzelner Bits in einem
'Register.
'Das Listing war brigens nicht im FUNKAMATEUR-Artikel enthalten, weil die 3 Seiten bereits voll waren...
'
'Hardware:  - AVR Butterfly
'           - spezieller Programmieradapter ("Adapter-Adapter")
'Funktion:  Ausgabe auf der LCD-Anzeige
'-----------------------------------------------------------------------------------------------------------------------------
$Regfile="m16def.dat"   'Deklarationen fr den ATmega16, weil es die fr den ATmega169 noch nicht gibt
$Crystal=1000000        'Interner RC-Oszillator mit 1 MHz

'Die folgenden Register-Deklarationen wurden genau so aus der Include-Datei fr den ATmega169 entnommen.
'Bitten beachten Sie, da die hier verwendeten Registeradressen alle ber &H3F liegen und damit keine
'I/O-Adressen sein knnen - es sind SRAM-Adressen. Das hat Atmel so gemacht, weil der I/O-Adrebereich
'fr die vielen Register nicht ausreichte.
'Zum einen finden Sie in der Include-Datei Register mit normalen I/O-Adressen im Bereich von &H00 bis &H3F.
'Will man auf diese Register mit den Basic-Befehlen INP() und OUT zugreifen oder (wie hier in diesem
'Listing) ber Overlay-Variablen, dann mu man &H20 zur Register-Adresse hinzuaddieren.
'Auf die Register, die in der Include-Datei mit Adressen ab &H40 aufwrts angegeben sind, kann man mit
'Overlay-Variablen oder den Basic-Befehlen INP() und OUT ohne Adre-Umrechnung zugreifen.
Dim LCDCCR As Byte At &HE7 Overlay     'LCD Contrast Control Register
Dim LCDFRR As Byte At &HE6 Overlay     'LCD Frame Rate Register
Dim LCDCRB As Byte At &HE5 Overlay     'LCD Control And Status Register B
Dim LCDCRA As Byte At &HE4 Overlay     'LCD Control and Status Register A
Dim LCDDR0 As Byte At &HEC Overlay     'LCD Driver Register 0
Dim LCDDR1 As Byte At &HED Overlay     'LCD Driver Register 1
'Dim SRAM_DDRA As Byte At &H21 Overlay 'Beispiel fr ein Register im Bereich &H00...&H3F; &H21=&H01+&H20
'Dim SRAM_DDRC As Byte At &H27 Overlay 'Beispiel fr ein Register im Bereich &H00...&H3F; &H27=&H07+&H20
'-----------------------------------------------------------------------------------------------------------------------------
'LCD-Anzeige initialisieren:

'32KHz-Oscillator whlen, Bias auf 1/3, Duty auf 1/3 setzen, LCD-Port-Maske auf SEG0 bis SEG20:
LCDCRB=&B10100100     'LCD Control And Status Register B, gesetzte Bits: LCDCS, LCDMUX1, LCDPM2
'Vorteilung des (32KHz-) Takts durch 16 und noch einmal durch 7. Das ergibt eine LCD-Taktrate von 49 Hz:
LCDFRR=&B00000110     'LCD Frame Rate Register, gesetzte Bits: LCDCD2, LCDCD1
'Ausgangsspannung zur LCD-Anzeige auf 3.3 V setzen:
LCDCCR=&B00001110     'LCD Contrast Control Register, gesetzte Bits: LCDCC3, LCDCC2, LCDCC1
'LCD-Anzeige einschalten, keine Interrupts, normale (keine LowPower-) WaveForm:
LCDCRA=&B10000000     'LCD Control and Status Register A, gesetztes Bit: LCDEN
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'LCD-Darstellung:
'
'Jetzt schalten wir einige LCD-Segmente ein. Ganz oben auf der LCD-Anzeige des Butterfly
'sind ein paar kleine Ziffern als Einzelsegmente vorhanden und diese wollen wir darstellen.
'Die Zuordnung der LCD-Segmente zu den AVR-Pins ist in der Dokumentations-PDF zum STK502
'zu finden (siehe www.atmel.com).
LCDDR0=&B01000100     'LCDDR0...LCDDR19: LCD-Ausgaberegister, die "1" und die "2" einschalten
LCDDR1=&B00100010     'Die "4" und die "5" einschalten

Do
Loop